@@ -22,6 +22,7 @@
  * Contributor(s):
  *   Roland Mainz <roland.mainz@informatik.med.uni-giessen.de>
  *   Fredrik Holmqvist <thesuckiestemail@yahoo.se>
+ *   Masayuki Nakano <masayuki@d-toybox.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -92,6 +93,9 @@
 #include "nsILocalFile.h"
 #include "nsILookAndFeel.h"
 #include "nsIProcess.h"
+#include "nsIIOService.h"
+#include "nsIJARURI.h"
+#include "nsNetCID.h"
 
 #ifdef MOZ_XPINSTALL
 #include "InstallCleanupDefines.h"
@@ -541,12 +545,8 @@ LaunchApplicationWithArgs(const char *commandLineArg,
 
   nsresult rv;
 
-  nsCOMPtr<nsICmdLineService> cmdLine =
-    do_GetService(NS_COMMANDLINESERVICE_CONTRACTID, &rv);
-  if (NS_FAILED(rv)) return rv;
-
   nsCOMPtr <nsICmdLineHandler> handler;
-  rv = cmdLine->GetHandlerForParam(aParam, getter_AddRefs(handler));
+  rv = cmdLineArgs->GetHandlerForParam(aParam, getter_AddRefs(handler));
   if (NS_FAILED(rv)) return rv;
 
   if (!handler) return NS_ERROR_FAILURE;
@@ -576,13 +576,32 @@ LaunchApplicationWithArgs(const char *commandLineArg,
         if (NS_FAILED(rv)) return rv;
 
         if (openWindowWithArgs) {
-          NS_ConvertASCIItoUTF16 cmdArgs(cmdResult);
+          nsAutoString cmdArgs;
+          NS_CopyNativeToUnicode(cmdResult, cmdArgs);
 #ifdef DEBUG_CMD_LINE
           printf("opening %s with %s\n", chromeUrlForTask.get(), "OpenWindow");
 #endif /* DEBUG_CMD_LINE */
           rv = OpenWindow(chromeUrlForTask, cmdArgs);
         }
         else {
+          // We need to emulate checkLoadURI because we don't have a window yet
+          nsCOMPtr<nsIIOService> ioService = do_GetService(NS_IOSERVICE_CONTRACTID, &rv);
+          if (NS_FAILED(rv)) return rv;
+          nsCOMPtr<nsIURI> baseURI;
+          rv = ioService->NewURI(cmdResult, nsnull, nsnull, getter_AddRefs(baseURI));
+          if (NS_FAILED(rv)) return rv;
+          nsCOMPtr<nsIJARURI> jarURI(do_QueryInterface(baseURI));
+          while (jarURI) {
+            rv = jarURI->GetJARFile(getter_AddRefs(baseURI));
+            if (NS_FAILED(rv)) return rv;
+            jarURI = do_QueryInterface(baseURI);
+          }
+          PRBool scriptOrData = PR_FALSE;
+          baseURI->SchemeIs("javascript", &scriptOrData);
+          if (!scriptOrData)
+            baseURI->SchemeIs("data", &scriptOrData);
+          if (scriptOrData)
+            return NS_ERROR_FAILURE;
 #ifdef DEBUG_CMD_LINE
           printf("opening %s with %s\n", cmdResult.get(), "OpenWindow");
 #endif /* DEBUG_CMD_LINE */
@@ -646,7 +665,7 @@ static PRBool IsStartupCommand(const char *arg)
 
 
 // This should be done by app shell enumeration someday
-nsresult DoCommandLines(nsICmdLineService* cmdLineArgs, PRBool heedGeneralStartupPrefs, PRBool *windowOpened)
+nsresult DoCommandLines(nsICmdLineService* cmdLineArgs, PRBool *windowOpened)
 {
   NS_ENSURE_ARG(windowOpened);
   *windowOpened = PR_FALSE;
@@ -667,21 +686,18 @@ nsresult DoCommandLines(nsICmdLineService* cmdLineArgs, PRBool heedGeneralStartu
 	if (NS_SUCCEEDED(rv) && !tempString.IsEmpty())
     PR_sscanf(tempString.get(), "%d", &height);
   
-  if (heedGeneralStartupPrefs) {
-    nsCOMPtr<nsIAppStartup> appStartup(do_GetService(NS_APPSTARTUP_CONTRACTID, &rv));
-    if (NS_FAILED(rv)) return rv;
-    rv = appStartup->CreateStartupState(width, height, windowOpened);
-    if (NS_FAILED(rv)) return rv;
-  }
-  else {
     PRInt32 argc = 0;
     rv = cmdLineArgs->GetArgc(&argc);
-    if (NS_FAILED(rv)) return rv;
+  if (NS_FAILED(rv)) 
+    return rv;
 
     char **argv = nsnull;
     rv = cmdLineArgs->GetArgv(&argv);
-    if (NS_FAILED(rv)) return rv;
+  if (NS_FAILED(rv)) 
+    return rv;
 
+  // first apply command line args
+  // if there is any window opened, startup preferences are ignored
     PRInt32 i = 0;
     for (i=1;i<argc;i++) {
 #ifdef DEBUG_CMD_LINE
@@ -706,6 +722,37 @@ nsresult DoCommandLines(nsICmdLineService* cmdLineArgs, PRBool heedGeneralStartu
           return rv;
       }
     }
+   // second if an URL is given in the command line
+   // then open a browser window with the URL
+  nsXPIDLCString urlToLoad;
+  rv = cmdLineArgs->GetURLToLoad(getter_Copies(urlToLoad));
+  if (NS_SUCCEEDED(rv) && !urlToLoad.IsEmpty()) {
+    nsCOMPtr<nsICmdLineHandler> handler(
+      do_GetService("@mozilla.org/commandlinehandler/general-startup;1?type=browser"));
+
+    nsXPIDLCString chromeUrlForTask;
+    rv = handler->GetChromeUrlForTask(getter_Copies(chromeUrlForTask));
+
+    if (NS_SUCCEEDED(rv)) {
+      // convert the cmdLine URL to Unicode
+      nsAutoString url;
+      NS_CopyNativeToUnicode(urlToLoad, url);
+      rv = OpenWindow(chromeUrlForTask, url, width, height);
+    }
+
+    if (NS_SUCCEEDED(rv)) {
+      *windowOpened = PR_TRUE;
+    }
+  }
+
+  // third if no window is opened then apply the startup preferences
+  if (!*windowOpened) {
+    nsCOMPtr<nsIAppStartup> appStartup(do_GetService(NS_APPSTARTUP_CONTRACTID, &rv));
+    if (NS_FAILED(rv))
+      return rv;
+    rv = appStartup->CreateStartupState(width, height, windowOpened);
+    if (NS_FAILED(rv))
+      return rv;
   }
   return NS_OK;
 }
@@ -1159,35 +1206,9 @@ static nsresult main1(int argc, char* argv[], nsISupports *nativeApp )
   NS_TIMELINE_LEAVE("appStartup->CreateHiddenWindow");
 
   // This will go away once Components are handling there own commandlines
-  // if we have no command line arguments, we need to heed the
-  // "general.startup.*" prefs
-  // if we had no command line arguments, argc == 1.
 
   PRBool windowOpened = PR_FALSE;
-  PRBool defaultStartup;
-#if defined(XP_MAC) || defined(XP_MACOSX)
-  // On Mac, nsCommandLineServiceMac may have added synthetic
-  // args. Check this adjusted value instead of the raw value.
-  PRInt32 processedArgc;
-  cmdLineArgs->GetArgc(&processedArgc);
-  defaultStartup = (processedArgc == 1);
-#if defined(XP_MACOSX)
-  // On OSX, we get passed two args if double-clicked from the Finder.
-  // The second is our PSN. Check for this and consider it to be default.
-  if (argc == 2 && processedArgc == 2) {
-    ProcessSerialNumber ourPSN;
-    if (::MacGetCurrentProcess(&ourPSN) == noErr) {
-      char argBuf[64];
-      sprintf(argBuf, "-psn_%ld_%ld", ourPSN.highLongOfPSN, ourPSN.lowLongOfPSN);
-      if (!strcmp(argBuf, argv[1]))
-        defaultStartup = PR_TRUE;
-    }
-  }
-#endif /* XP_MACOSX */
-#else
-  defaultStartup = (argc == 1);
-#endif
-  rv = DoCommandLines(cmdLineArgs, defaultStartup, &windowOpened);
+  rv = DoCommandLines(cmdLineArgs, &windowOpened);
   if (NS_FAILED(rv))
   {
     NS_WARNING("failed to process command line");
@@ -1356,7 +1377,7 @@ static nsresult DumpVersion(char *appname)
   nsresult rv = NS_OK;
   long buildID = NS_BUILD_ID;  // 10-digit number
 
-  printf("%s %s, Copyright (c) 2003-2006 mozilla.org", NS_STRINGIFY(MOZ_APP_DISPLAYNAME), NS_STRINGIFY(MOZ_APP_VERSION));
+  printf("%s %s, Copyright (c) 2003-2008 mozilla.org", NS_STRINGIFY(MOZ_APP_DISPLAYNAME), NS_STRINGIFY(MOZ_APP_VERSION));
 
   if(buildID) {
     printf(", build %u\n", (unsigned int)buildID);
@@ -1554,6 +1575,11 @@ public:
 
 int main(int argc, char* argv[])
 {
+#ifdef XP_WIN32
+  if (argc > 4 && !strcmp(argv[1], "-osint"))
+    return 1;
+#endif
+
   NS_TIMELINE_MARK("enter main");
   int i; //Moved here due to portability guideline 20. See bug 258055
 
@@ -1576,6 +1602,20 @@ int main(int argc, char* argv[])
   InstallUnixSignalHandlers(argv[0]);
 #endif
 
+#ifdef MOZ_ACCESSIBILITY_ATK
+  // Reset GTK_MODULES, strip atk-bridge if exists
+  // Mozilla will load libatk-bridge.so later if necessary
+  const char* gtkModules = PR_GetEnv("GTK_MODULES");
+  if (gtkModules && *gtkModules) {
+    nsCString gtkModulesStr(gtkModules);
+    gtkModulesStr.ReplaceSubstring("atk-bridge", "");
+    char* expr = PR_smprintf("GTK_MODULES=%s", gtkModulesStr.get());
+    if (expr)
+      PR_SetEnv(expr);
+    // We intentionally leak |expr| here since it is required by PR_SetEnv.
+  }
+#endif
+
 #if defined(XP_OS2)
   __pargc = &argc;
   __argv = argv;
@@ -1629,6 +1669,9 @@ int main(int argc, char* argv[])
   // in nsAppShell::Create, but we need to get in before gtk
   // has been initialized to make sure everything is running
   // consistently.
+#if defined(MOZ_WIDGET_GTK2)
+  g_thread_init(NULL);
+#endif
   for (i=1; i<argc; i++)
     if ((PL_strcasecmp(argv[i], "-install") == 0)
         || (PL_strcasecmp(argv[i], "--install") == 0)) {
